#include "nemo-video-thumbnailer.h"

#include <glib.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <unistd.h>
#include <libffmpegthumbnailer/videothumbnailerc.h>
#include <gdk/gdk.h>


static gboolean check_file_existed (const char *file_name)
{
    if (!access (file_name, F_OK))
        return TRUE;

    return FALSE;
}

// 获取单个视频文件首帧缩略图
void generate_video_thumbnail (const char *movie_file, const char *thumbnail_file, int size)
{
    video_thumbnailer *thumbnailer = video_thumbnailer_create();
    if (!thumbnailer)
    {
        g_warning ("Failed to create video thumbnailer");
        goto out;
    }

    if (size > 0)
        thumbnailer->thumbnail_size = size;

    video_thumbnailer_generate_thumbnail_to_file (thumbnailer, movie_file, thumbnail_file);

    // 检查缩略图是否生成
    if (!check_file_existed (thumbnail_file))
    {
        g_warning ("Failed to  generate video %s thumbnail", movie_file);
        goto out;
    }

out:
    if (thumbnailer)
        video_thumbnailer_destroy (thumbnailer);
}

// 获取单个视频文件GdkPixbuf实例
GdkPixbuf *generate_video_thumbnail_pixbuf (const char *uri, int size)
{
    GdkPixbuf *pixbuf = NULL;
    video_thumbnailer* thumbnailer;
    image_data *data = NULL;
    GFile *file = NULL;
    gchar *file_path = NULL;
    GInputStream *stream = NULL;
    GError *err = NULL;

    file = g_file_new_for_uri (uri);
    // 共享目录文件也显示缩略图，不然每次进入共享目录的视频文件目录时，都会不断执行这个函数，不断报错，视频文件图标一直显示为image-loading
    file_path = g_file_get_path (file);

    data = video_thumbnailer_create_image_data ();
    thumbnailer = video_thumbnailer_create();
    if (size > 0)
        thumbnailer->thumbnail_size = size;

    if (video_thumbnailer_generate_thumbnail_to_buffer (thumbnailer, file_path, data) != 0) {
        g_warning ("Failed to generate thumbnail buffer");
        goto out;
    }

    stream = g_memory_input_stream_new_from_data (data->image_data_ptr,
                                                                                            data->image_data_size,
                                                                                            NULL);
    if (!stream) {
        g_warning ("Failed to get image data stream");
        goto out;
    }

    pixbuf = gdk_pixbuf_new_from_stream (stream, NULL, &err);
    if (!pixbuf) {
        g_warning ("Failed to get gdk pixbuf  from stream");
        goto out;
    }

out:
    if (file)
        g_object_unref (file);
    if (data)
        video_thumbnailer_destroy_image_data (data);
    if (file_path)
        g_free (file_path);
    if (thumbnailer)
        video_thumbnailer_destroy (thumbnailer);
    if (stream)
        g_object_unref (stream);
    
    return pixbuf;
}
